#!/usr/bin/env python3

import numpy as np
from Controllers.Parsers.Parser.Bias import Bias


def load(obj, parsedNetworkObj):
    # This operation supports only one inputTensor
    if (len(obj.bottom) != 1):
        throw_error(ErrorTable.StageDetailsNotSupported,
            "Bias layer {0}: Unsupported number of input blobs. Max number of input " +
            "blobs = 1 got {1}".format(obj.name, len(obj.bottom)))

    if (obj.scale_param.axis != 1):
        throw_error(ErrorTable.StageDetailsNotSupported,
            "Scale layer {0}: Unsupported axis parameter value." +
            "Parmeter axis must be in = {{1, -3}} got {1}".format(
                obj.name, obj.scale_param.axis))

    if (obj.scale_param.num_axes != 1):
        throw_error(ErrorTable.StageDetailsNotSupported,
            "Scale layer {0}: Unsupported num_axes parameter value." +
            "Parmeter axes must be = 1 got {1}".format(
                obj.name, obj.scale_param.num_axes))

    x = Bias(obj.name, obj.bottom, obj.top)

    trainedParameters = tuple(parsedNetworkObj.params[obj.name])

    x.loadBias(trainedParameters[0].data.astype(dtype=np.float16))

    return [x]
