# Copyright 2018 Intel Corporation.
# The source code, information and material ("Material") contained herein is
# owned by Intel Corporation or its suppliers or licensors, and title to such
# Material remains with Intel Corporation or its suppliers or licensors.
# The Material contains proprietary information of Intel or its suppliers and
# licensors. The Material is protected by worldwide copyright laws and treaty
# provisions.
# No part of the Material may be used, copied, reproduced, modified, published,
# uploaded, posted, transmitted, distributed or disclosed in any way without
# Intel's prior express written permission. No license under any patent,
# copyright or other intellectual property rights in the Material is granted to
# or conferred upon you, either expressly, by implication, inducement, estoppel
# or otherwise.
# Any license under such intellectual property rights must be express and
# approved by Intel in writing.


from .Layer import Layer
import Models.Layouts as Layouts
from Controllers.TensorFormat import LayerIOFormat, LayerIOFormatPair
from Models.EnumDeclarations import PadStyle

from enum import Enum

class Pooling(Layer):
    _supported_IO_formats = [
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHWC_to_NHWC_W_stride),
        ]

    class Type(Enum):
        MAX = 'Maximum'
        AVE = 'Average'

    def __init__(self, *args):
        super().__init__(*args)
        self.padStyle = PadStyle.caffe

    def getType(self):
        return self.type

    def loadType(self, type):
        self.type = type

    def loadGlobal(self, flag):
        self.globalPooling = flag

    def isGlobal(self):
        return self.globalPooling

    def getKernelSize(self):
        return (self.kernelHeight, self.kernelWidth)

    def getPadding(self):
        return (self.pad_top, self.pad_bottom, self.pad_left, self.pad_right)

    def getStride(self):
        return (self.strideHeight, self.strideWidth)

    def loadKernelSize(self, kernelHeight, kernelWidth):
        self.kernelHeight = kernelHeight
        self.kernelWidth = kernelWidth

    def loadStrideSize(self, strideHeight, strideWidth):
        self.strideHeight = strideHeight
        self.strideWidth = strideWidth

    def loadPadding(self, pad_top, pad_bottom, pad_left, pad_right):
        self.pad_top    = pad_top
        self.pad_bottom = pad_bottom
        self.pad_left   = pad_left
        self.pad_right  = pad_right

    def force_symmetrical_padding(self):
        if self.padStyle == PadStyle.tfvalid or self.padStyle == PadStyle.tfsame:
            self.padStyle = PadStyle.caffe
            self.pad_top    = self.pad_bottom
            self.pad_left   = self.pad_right

    def setPadStyle(self, padStyle):
        if padStyle == b'VALID':
            self.padStyle = PadStyle.tfvalid
        elif padStyle == b'SAME':
            self.padStyle = PadStyle.tfsame
