# Copyright 2018 Intel Corporation.
# The source code, information and material ("Material") contained herein is
# owned by Intel Corporation or its suppliers or licensors, and title to such
# Material remains with Intel Corporation or its suppliers or licensors.
# The Material contains proprietary information of Intel or its suppliers and
# licensors. The Material is protected by worldwide copyright laws and treaty
# provisions.
# No part of the Material may be used, copied, reproduced, modified, published,
# uploaded, posted, transmitted, distributed or disclosed in any way without
# Intel's prior express written permission. No license under any patent,
# copyright or other intellectual property rights in the Material is granted to
# or conferred upon you, either expressly, by implication, inducement, estoppel
# or otherwise.
# Any license under such intellectual property rights must be express and
# approved by Intel in writing.


from .Layer import Layer
import numpy as np

import Models.Layouts as Layouts
from Controllers.TensorFormat import LayerIOFormat, LayerIOFormatPair

class Scale(Layer):
    _supported_IO_formats = [
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHWC_to_NHWC_W_stride),

        # NHCW_to_NHCW_C_stride_8 IOFormat parir is used even though Scale layer
        # MvTensor implementation does not require an EnclosureRequest on the W axis
        # and it can work with any stride. Usage of NHCW_to_NHCW_C_stride_8 is a
        # hack/workaround in order to accomodate the usecase of TensorFlow tiny_yolo_v2
        # where the Scale output enters an Elementwise along with a HW layer output
        # and workaround the postOp<->isRegularOp madness.
        # TODO: Once the postOp<->isRegularOp is sorted out this format pair should
        # changed to: LayerIOFormatPair.NHCW_to_NHCW_C_stride
        LayerIOFormat.getLayerIOFormatPair(LayerIOFormatPair.NHCW_to_NHCW_C_stride_8),
        ]

    def __init__(self, *args):
        super().__init__(*args)

        self.multiplier = None
        self.bias = None

    def loadMultiplier(self, multiplier):
        self.multiplier = multiplier

    def loadBias(self, bias):
        self.bias = bias

    def getBias(self):
        return self.bias

    def getMultiplier(self):
        return self.multiplier
