# Copyright 2018 Intel Corporation.
# The source code, information and material ("Material") contained herein is
# owned by Intel Corporation or its suppliers or licensors, and title to such
# Material remains with Intel Corporation or its suppliers or licensors.
# The Material contains proprietary information of Intel or its suppliers and
# licensors. The Material is protected by worldwide copyright laws and treaty
# provisions.
# No part of the Material may be used, copied, reproduced, modified, published,
# uploaded, posted, transmitted, distributed or disclosed in any way without
# Intel's prior express written permission. No license under any patent,
# copyright or other intellectual property rights in the Material is granted to
# or conferred upon you, either expressly, by implication, inducement, estoppel
# or otherwise.
# Any license under such intellectual property rights must be express and
# approved by Intel in writing.


import numpy as np

from Controllers.Parsers.Parser.Scale import Scale
from Controllers.EnumController import throw_error
from Models.EnumDeclarations import ErrorTable

def load(obj, parsedNetworkObj):
    # This operation supports only one inputTensor
    if (len(obj.bottom) != 1):
        throw_error(ErrorTable.StageDetailsNotSupported,
            "Scale layer {0}: Unsupported number of input blobs. Max number of input " +
            "blobs = 1 got {1}".format(obj.name, len(obj.bottom)))

    if (obj.scale_param.axis != 1):
        throw_error(ErrorTable.StageDetailsNotSupported,
            "Scale layer {0}: Unsupported axis parameter value." +
            "Parmeter axis must be in = {{1, -3}} got {1}".format(
                obj.name, obj.scale_param.axis))

    if (obj.scale_param.num_axes != 1):
        throw_error(ErrorTable.StageDetailsNotSupported,
            "Scale layer {0}: Unsupported num_axes parameter value." +
            "Parmeter axes must be = 1 got {1}".format(
                obj.name, obj.scale_param.num_axes))

    x = Scale(obj.name, obj.bottom, obj.top)

    trainedParameters = tuple(parsedNetworkObj.params[obj.name])

    assert(len(trainedParameters) > 0 and len(trainedParameters) <= 2)

    x.loadMultiplier(trainedParameters[0].data.astype(dtype=np.float16))
    if len(trainedParameters) == 2:
        x.loadBias(trainedParameters[1].data.astype(dtype=np.float16))

    return [x]